"""Models for Docker API."""

from __future__ import annotations

from dataclasses import dataclass, field
from typing import Any

from mashumaro import field_options
from mashumaro.mixins.orjson import DataClassORJSONMixin


@dataclass
class ImageManifestDescriptorPlatform(DataClassORJSONMixin):
    """Represents the platform information of an image manifest descriptor."""

    architecture: str | None = None
    os: str | None = None
    variant: str | None = None
    os_version: str | None = field(default=None, metadata=field_options(alias="os.version"))
    os_features: list[str] | None = field(default=None, metadata=field_options(alias="os.features"))


@dataclass
class ImageManifestDescriptor(DataClassORJSONMixin):
    """Represents an image manifest descriptor."""

    digest: str | None = None
    size: int | None = None
    urls: list[str] | None = None
    annotations: dict[str, str] | None = None
    data: Any | None = None
    platform: ImageManifestDescriptorPlatform | None = None
    media_type: str | None = field(default=None, metadata=field_options(alias="mediaType"))
    artifact_type: Any | None = field(default=None, metadata=field_options(alias="artifactType"))


@dataclass
class Port(DataClassORJSONMixin):
    """Represents a port mapping for a Docker container."""

    private_port: int | None = field(default=None, metadata=field_options(alias="PrivatePort"))
    public_port: int | None = field(default=None, metadata=field_options(alias="PublicPort"))
    type: str | None = field(default=None, metadata=field_options(alias="Type"))


@dataclass
class HostConfig(DataClassORJSONMixin):
    """Represents the host configuration for a Docker container."""

    annotations: dict[str, str] | None = None
    network_mode: str | None = field(default=None, metadata=field_options(alias="NetworkMode"))


@dataclass
class IPAMConfig(DataClassORJSONMixin):
    """Represents the IP Address Management (IPAM) configuration for a Docker container."""

    ipv4_address: str | None = field(default=None, metadata=field_options(alias="IPv4Address"))
    ipv6_address: str | None = field(default=None, metadata=field_options(alias="IPv6Address"))
    link_local_ips: list[str] | None = field(default=None, metadata=field_options(alias="LinkLocalIPs"))


@dataclass
class Network(DataClassORJSONMixin):
    """Represents the network configuration for a Docker container."""

    endpoint_id: str = field(metadata=field_options(alias="EndpointID"))

    links: list[str] | None = field(default=None, metadata=field_options(alias="Links"))
    aliases: list[str] | None = field(default=None, metadata=field_options(alias="Aliases"))
    gateway: str | None = field(default=None, metadata=field_options(alias="Gateway"))
    ipam_config: IPAMConfig | None = field(default=None, metadata=field_options(alias="IPAMConfig"))
    mac_address: str | None = field(default=None, metadata=field_options(alias="MacAddress"))
    driver_opts: dict[str, str] | None = field(default=None, metadata=field_options(alias="DriverOpts"))
    network_id: str | None = field(default=None, metadata=field_options(alias="NetworkID"))
    ip_address: str | None = field(default=None, metadata=field_options(alias="IPAddress"))
    ip_prefix_len: int | None = field(default=None, metadata=field_options(alias="IPPrefixLen"))
    ipv6_gateway: str | None = field(default=None, metadata=field_options(alias="IPv6Gateway"))
    global_ipv6_address: str | None = field(default=None, metadata=field_options(alias="GlobalIPv6Address"))
    global_ipv6_prefix_len: int | None = field(default=None, metadata=field_options(alias="GlobalIPv6PrefixLen"))
    dns_names: list[str] | None = field(default=None, metadata=field_options(alias="DNSNames"))


@dataclass
class NetworkSettings(DataClassORJSONMixin):
    """Represents the network settings for a Docker container."""

    networks: dict[str, Network] | None = field(default=None, metadata=field_options(alias="Networks"))


@dataclass
class Mount(DataClassORJSONMixin):
    """Represents a mount point for a Docker container."""

    type: str | None = field(default=None, metadata=field_options(alias="Type"))
    name: str | None = field(default=None, metadata=field_options(alias="Name"))
    source: str | None = field(default=None, metadata=field_options(alias="Source"))
    destination: str | None = field(default=None, metadata=field_options(alias="Destination"))
    driver: str | None = field(default=None, metadata=field_options(alias="Driver"))
    mode: str | None = field(default=None, metadata=field_options(alias="Mode"))
    rw: bool | None = field(default=None, metadata=field_options(alias="RW"))
    propagation: str | None = field(default=None, metadata=field_options(alias="Propagation"))


@dataclass
class DockerContainer(DataClassORJSONMixin):
    """Represents a Docker container."""

    id: str = field(metadata=field_options(alias="Id"))
    names: list[str] = field(default_factory=list, metadata=field_options(alias="Names"))

    image: str | None = field(default=None, metadata=field_options(alias="Image"))
    command: str | None = field(default=None, metadata=field_options(alias="Command"))
    created: str | None = field(default=None, metadata=field_options(alias="Created"))
    ports: list[Port] | None = field(default=None, metadata=field_options(alias="Ports"))
    labels: dict[str, str] | None = field(default=None, metadata=field_options(alias="Labels"))
    state: str | None = field(default=None, metadata=field_options(alias="State"))
    status: str | None = field(default=None, metadata=field_options(alias="Status"))
    mounts: list[Mount] | None = field(default=None, metadata=field_options(alias="Mounts"))

    image_id: str | None = field(default=None, metadata=field_options(alias="ImageID"))
    image_manifest_descriptor: ImageManifestDescriptor | None = field(default=None, metadata=field_options(alias="ImageManifestDescriptor"))
    size_rw: str | None = field(default=None, metadata=field_options(alias="SizeRw"))
    size_root_fs: str | None = field(default=None, metadata=field_options(alias="SizeRootFs"))
    host_config: HostConfig | None = field(default=None, metadata=field_options(alias="HostConfig"))
    network_settings: NetworkSettings | None = field(default=None, metadata=field_options(alias="NetworkSettings"))



"""Asynchronous Python client for Python Portainer."""

from __future__ import annotations

import asyncio
import socket
from dataclasses import dataclass
from importlib import metadata
from typing import Any, Self
from urllib.parse import urlparse

from aiohttp import ClientError, ClientResponseError, ClientSession
from aiohttp.hdrs import METH_GET
from yarl import URL

from pyportainer.exceptions import (
    PortainerAuthenticationError,
    PortainerConnectionError,
    PortainerError,
    PortainerNotFoundError,
    PortainerTimeoutError,
)
from pyportainer.models.docker import DockerContainer
from pyportainer.models.docker_inspect import DockerInfo, DockerInspect, DockerVersion
from pyportainer.models.portainer import Endpoint

try:
    VERSION = metadata.version(__package__)
except metadata.PackageNotFoundError:
    VERSION = "DEV-0.0.0"


@dataclass
class Portainer:
    """Main class for handling connections with the Python Portainer API."""

    request_timeout: float = 10.0
    session: ClientSession | None = None

    _close_session: bool = False

    def __init__(
        self,
        api_url: str,
        api_key: str,
        *,
        request_timeout: float = 10.0,
        session: ClientSession | None = None,
    ) -> None:
        """Initialize the Portainer object.

        Args:
        ----
            api_url: URL of the Portainer API.
            api_key: API key for authentication.
            request_timeout: Timeout for requests (in seconds).
            session: Optional aiohttp session to use.

        """
        self._api_key = api_key
        self._request_timeout = request_timeout
        self._session = session

        parsed_url = urlparse(api_url)
        self._api_host = parsed_url.hostname or "localhost"
        self._api_scheme = parsed_url.scheme or "http"
        self._api_port = parsed_url.port or 9000

    async def _request(
        self,
        uri: str,
        *,
        method: str = METH_GET,
        params: dict[str, Any] | None = None,
    ) -> Any:
        """Handle a request to the Python Portainer API.

        Args:
        ----
            uri: Request URI, without '/api/', for example, 'status'.
            method: HTTP method to use.
            params: Extra options to improve or limit the response.

        Returns:
        -------
            A Python dictionary (JSON decoded) with the response from
            the Python Portainer API.

        Raises:
        ------
            Python PortainerAuthenticationError: If the API key is invalid.

        """
        url = URL.build(
            scheme=self._api_scheme,
            host=self._api_host,
            port=self._api_port,
            path="/api/",
        ).join(URL(uri))

        headers = {
            "Accept": "application/json, text/plain",
            "User-Agent": f"PythonPortainer/{VERSION}",
            "X-API-Key": self._api_key,
        }

        if self._session is None:
            self._session = ClientSession()
            self._close_session = True

        try:
            async with asyncio.timeout(self._request_timeout):
                response = await self._session.request(
                    method,
                    url,
                    headers=headers,
                    params=params,
                )
                response.raise_for_status()
        except TimeoutError as err:
            msg = f"Timeout error while accessing {method} {url}: {err}"
            raise PortainerTimeoutError(msg) from err
        except ClientResponseError as err:
            if err.status == 401:
                msg = f"Authentication failed for {method} {url}: Invalid API key"
                raise PortainerAuthenticationError(msg) from err
            if err.status == 404:
                msg = f"Resource not found at {method} {url}: {err}"
                raise PortainerNotFoundError(msg) from err
            msg = f"Connection error for {method} {url}: {err}"
            raise PortainerConnectionError(msg) from err
        except (ClientError, socket.gaierror) as err:
            msg = f"Unexpected error during {method} {url}: {err}"
            raise PortainerConnectionError(msg) from err

        if response.status in (204, 304):
            return None

        content_type = response.headers.get("Content-Type", "")
        if "application/json" not in content_type:
            text = await response.text()
            msg = "Unexpected content type response from the Portainer API"
            raise PortainerError(
                msg,
                {"Content-Type": content_type, "response": text},
            )

        return await response.json()

    async def get_endpoints(self) -> list[Endpoint]:
        """Get the list of endpoints from the Portainer API.

        Returns
        -------
            A list of Endpoint objects.

        """
        endpoints = await self._request("endpoints")

        return [Endpoint.from_dict(endpoint) for endpoint in endpoints]

    async def get_containers(self, endpoint_id: int) -> list[DockerContainer]:
        """Get the list of containers from the Portainer API.

        Args:
        ----
            endpoint_id: The ID of the endpoint to get containers from.

        Returns:
        -------
            A list of containers.

        """
        containers = await self._request(f"endpoints/{endpoint_id}/docker/containers/json")

        return [DockerContainer.from_dict(container) for container in containers]

    async def start_container(self, endpoint_id: int, container_id: str) -> Any:
        """Start a container on the specified endpoint.

        Args:
        ----
            endpoint_id: The ID of the endpoint.
            container_id: The ID of the container to start.

        """
        return await self._request(
            f"endpoints/{endpoint_id}/docker/containers/{container_id}/start",
            method="POST",
        )

    async def stop_container(self, endpoint_id: int, container_id: str) -> Any:
        """Stop a container on the specified endpoint.

        Args:
        ----
            endpoint_id: The ID of the endpoint.
            container_id: The ID of the container to stop.

        """
        return await self._request(
            f"endpoints/{endpoint_id}/docker/containers/{container_id}/stop",
            method="POST",
        )

    async def restart_container(self, endpoint_id: int, container_id: str) -> Any:
        """Restart a container on the specified endpoint.

        Args:
        ----
            endpoint_id: The ID of the endpoint.
            container_id: The ID of the container to restart.

        """
        return await self._request(
            f"endpoints/{endpoint_id}/docker/containers/{container_id}/restart",
            method="POST",
        )

    async def pause_container(self, endpoint_id: int, container_id: str) -> Any:
        """Pause a container on the specified endpoint.

        Args:
        ----
            endpoint_id: The ID of the endpoint.
            container_id: The ID of the container to pause.

        """
        return await self._request(
            f"endpoints/{endpoint_id}/docker/containers/{container_id}/pause",
            method="POST",
        )

    async def unpause_container(self, endpoint_id: int, container_id: str) -> Any:
        """Unpause a container on the specified endpoint.

        Args:
        ----
            endpoint_id: The ID of the endpoint.
            container_id: The ID of the container to unpause.

        """
        return await self._request(
            f"endpoints/{endpoint_id}/docker/containers/{container_id}/unpause",
            method="POST",
        )

    async def kill_container(self, endpoint_id: int, container_id: str) -> Any:
        """Kill a container on the specified endpoint.

        Args:
        ----
            endpoint_id: The ID of the endpoint.
            container_id: The ID of the container to kill.

        """
        return await self._request(
            f"endpoints/{endpoint_id}/docker/containers/{container_id}/kill",
            method="POST",
        )

    async def inspect_container(self, endpoint_id: int, container_id: str) -> DockerInspect:
        """Inspect a container on the specified endpoint.

        Args:
        ----
            endpoint_id: The ID of the endpoint.
            container_id: The ID of the container to inspect.

        Returns:
        -------
            A DockerContainer object with the inspected data.

        """
        container = await self._request(f"endpoints/{endpoint_id}/docker/containers/{container_id}/json")

        return DockerInspect.from_dict(container)

    async def docker_version(self, endpoint_id: int) -> DockerVersion:
        """Get the Docker version on the specified endpoint.

        Args:
        ----
            endpoint_id: The ID of the endpoint.

        Returns:
        -------
            A DockerVersion object with the Docker version data.

        """
        version = await self._request(f"endpoints/{endpoint_id}/docker/version")

        return DockerVersion.from_dict(version)

    async def docker_info(self, endpoint_id: int) -> DockerInfo:
        """Get the Docker info on the specified endpoint.

        Args:
        ----
            endpoint_id: The ID of the endpoint.

        Returns:
        -------
            A DockerInfo object with the Docker info data.

        """
        info = await self._request(f"endpoints/{endpoint_id}/docker/info")

        return DockerInfo.from_dict(info)

    async def close(self) -> None:
        """Close open client session."""
        if self._session and self._close_session:
            await self._session.close()

    async def __aenter__(self) -> Self:
        """Async enter.

        Returns
        -------
            The Portainer object.

        """
        return self

    async def __aexit__(self, *_exc_info: object) -> None:
        """Async exit.

        Args:
        ----
            _exc_info: Exec type.

        """
        await self.close()



"""Model for Docker container inspection."""

from __future__ import annotations

from dataclasses import dataclass, field
from typing import Any

from mashumaro import field_options
from mashumaro.mixins.orjson import DataClassORJSONMixin


@dataclass
class HealthLog(DataClassORJSONMixin):
    """Represents a health log entry for a Docker container."""

    start: str | None = field(default=None, metadata=field_options(alias="Start"))
    end: str | None = field(default=None, metadata=field_options(alias="End"))
    exit_code: int | None = field(default=None, metadata=field_options(alias="ExitCode"))
    output: str | None = field(default=None, metadata=field_options(alias="Output"))


@dataclass
class Health(DataClassORJSONMixin):
    """Represents the health status of a Docker container."""

    status: str | None = field(default=None, metadata=field_options(alias="Status"))
    failing_streak: int | None = field(default=None, metadata=field_options(alias="FailingStreak"))
    log: list[HealthLog] | None = field(default=None, metadata=field_options(alias="Log"))


@dataclass
class State(DataClassORJSONMixin):
    """Represents the state of a Docker container."""

    status: str | None = field(default=None, metadata=field_options(alias="Status"))
    running: bool | None = field(default=None, metadata=field_options(alias="Running"))
    paused: bool | None = field(default=None, metadata=field_options(alias="Paused"))
    restarting: bool | None = field(default=None, metadata=field_options(alias="Restarting"))
    oom_killed: bool | None = field(default=None, metadata=field_options(alias="OOMKilled"))
    dead: bool | None = field(default=None, metadata=field_options(alias="Dead"))
    pid: int | None = field(default=None, metadata=field_options(alias="Pid"))
    exit_code: int | None = field(default=None, metadata=field_options(alias="ExitCode"))
    error: str | None = field(default=None, metadata=field_options(alias="Error"))
    started_at: str | None = field(default=None, metadata=field_options(alias="StartedAt"))
    finished_at: str | None = field(default=None, metadata=field_options(alias="FinishedAt"))
    health: Health | None = field(default=None, metadata=field_options(alias="Health"))


@dataclass
class ImageManifestDescriptorPlatform(DataClassORJSONMixin):
    """Represents the platform information of an image manifest descriptor."""

    architecture: str | None = None
    os: str | None = None
    variant: str | None = None
    os_version: str | None = field(default=None, metadata=field_options(alias="os.version"))
    os_features: list[str] | None = field(default=None, metadata=field_options(alias="os.features"))


@dataclass
class ImageManifestDescriptor(DataClassORJSONMixin):
    """Represents an image manifest descriptor."""

    media_type: str | None = field(default=None, metadata=field_options(alias="mediaType"))
    digest: str | None = None
    size: int | None = None
    urls: list[str] | None = None
    annotations: dict[str, str] | None = None
    data: Any | None = None
    platform: ImageManifestDescriptorPlatform | None = None
    artifact_type: Any | None = field(default=None, metadata=field_options(alias="artifactType"))


@dataclass
class GraphDriver(DataClassORJSONMixin):
    """Represents the graph driver information for a Docker container."""

    name: str | None = field(default=None, metadata=field_options(alias="Name"))
    data: dict[str, str] | None = field(default=None, metadata=field_options(alias="Data"))


@dataclass
class DockerInspect(DataClassORJSONMixin):
    """Represents the Docker container inspection data."""

    id: str | None = field(default=None, metadata=field_options(alias="Id"))
    created: str | None = field(default=None, metadata=field_options(alias="Created"))
    path: str | None = field(default=None, metadata=field_options(alias="Path"))
    args: list[str] | None = field(default=None, metadata=field_options(alias="Args"))
    state: State | None = field(default=None, metadata=field_options(alias="State"))
    image: str | None = field(default=None, metadata=field_options(alias="Image"))
    resolv_conf_path: str | None = field(default=None, metadata=field_options(alias="ResolvConfPath"))
    hostname_path: str | None = field(default=None, metadata=field_options(alias="HostnamePath"))
    hosts_path: str | None = field(default=None, metadata=field_options(alias="HostsPath"))
    log_path: str | None = field(default=None, metadata=field_options(alias="LogPath"))
    name: str | None = field(default=None, metadata=field_options(alias="Name"))
    restart_count: int | None = field(default=None, metadata=field_options(alias="RestartCount"))
    driver: str | None = field(default=None, metadata=field_options(alias="Driver"))
    platform: str | None = field(default=None, metadata=field_options(alias="Platform"))
    image_manifest_descriptor: ImageManifestDescriptor | None = field(default=None, metadata=field_options(alias="ImageManifestDescriptor"))
    mount_label: str | None = field(default=None, metadata=field_options(alias="MountLabel"))
    process_label: str | None = field(default=None, metadata=field_options(alias="ProcessLabel"))
    app_armor_profile: str | None = field(default=None, metadata=field_options(alias="AppArmorProfile"))
    exec_ids: list[str] | None = field(default=None, metadata=field_options(alias="ExecIDs"))
    graph_driver: GraphDriver | None = field(default=None, metadata=field_options(alias="GraphDriver"))


@dataclass
class DockerVersion(DataClassORJSONMixin):
    """Represents the Docker version information."""

    @dataclass
    class PlatformInfo(DataClassORJSONMixin):
        """Represents the platform information for Docker version."""

        name: str | None = field(default=None, metadata=field_options(alias="Name"))

    platform: PlatformInfo | None = field(default=None, metadata=field_options(alias="Platform"))

    version: str | None = field(default=None, metadata=field_options(alias="Version"))
    api_version: str | None = field(default=None, metadata=field_options(alias="ApiVersion"))
    min_api_version: str | None = field(default=None, metadata=field_options(alias="MinAPIVersion"))
    git_commit: str | None = field(default=None, metadata=field_options(alias="GitCommit"))
    go_version: str | None = field(default=None, metadata=field_options(alias="GoVersion"))
    os: str | None = field(default=None, metadata=field_options(alias="Os"))
    arch: str | None = field(default=None, metadata=field_options(alias="Arch"))
    kernel_version: str | None = field(default=None, metadata=field_options(alias="KernelVersion"))
    build_time: str | None = field(default=None, metadata=field_options(alias="BuildTime"))
    experimental: bool | None = field(default=None, metadata=field_options(alias="Experimental"))
    components: list[dict[str, Any]] | None = field(default=None, metadata=field_options(alias="Components"))
    warnings: list[str] | None = field(default=None, metadata=field_options(alias="Warnings"))


@dataclass
class CommitInfo(DataClassORJSONMixin):
    """Represents commit information for Docker components."""

    id: str | None = field(default=None, metadata=field_options(alias="ID"))
    expected: str | None = field(default=None, metadata=field_options(alias="Expected"))


@dataclass
class SwarmInfo(DataClassORJSONMixin):
    """Represents the Swarm mode information for a Docker daemon."""

    node_id: str | None = field(default=None, metadata=field_options(alias="NodeID"))
    node_addr: str | None = field(default=None, metadata=field_options(alias="NodeAddr"))
    local_node_state: str | None = field(default=None, metadata=field_options(alias="LocalNodeState"))
    control_available: bool | None = field(default=None, metadata=field_options(alias="ControlAvailable"))
    error: str | None = field(default=None, metadata=field_options(alias="Error"))
    remote_managers: list[Any] | None = field(default=None, metadata=field_options(alias="RemoteManagers"))
    nodes: int | None = field(default=None, metadata=field_options(alias="Nodes"))
    managers: int | None = field(default=None, metadata=field_options(alias="Managers"))
    cluster: dict[str, Any] | None = field(default=None, metadata=field_options(alias="Cluster"))


@dataclass
class RegistryConfig(DataClassORJSONMixin):
    """Represents the registry configuration for a Docker daemon."""

    allow_nondistributable_artifacts_cidrs: list[str] | None = field(
        default=None, metadata=field_options(alias="AllowNondistributableArtifactsCIDRs")
    )
    allow_nondistributable_artifacts_hostnames: list[str] | None = field(
        default=None, metadata=field_options(alias="AllowNondistributableArtifactsHostnames")
    )
    insecure_registry_cidrs: list[str] | None = field(default=None, metadata=field_options(alias="InsecureRegistryCIDRs"))
    index_configs: dict[str, Any] | None = field(default=None, metadata=field_options(alias="IndexConfigs"))
    mirrors: list[str] | None = field(default=None, metadata=field_options(alias="Mirrors"))


@dataclass
class PluginsInfo(DataClassORJSONMixin):
    """Represents the plugins information for a Docker daemon."""

    volume: list[str] | None = field(default=None, metadata=field_options(alias="Volume"))
    network: list[str] | None = field(default=None, metadata=field_options(alias="Network"))
    authorization: list[str] | None = field(default=None, metadata=field_options(alias="Authorization"))
    log: list[str] | None = field(default=None, metadata=field_options(alias="Log"))


@dataclass
class DockerInfo(DataClassORJSONMixin):  # pylint: disable=too-many-instance-attributes
    """Represents the Docker daemon information."""

    id: str | None = field(default=None, metadata=field_options(alias="ID"))
    containers: int | None = field(default=None, metadata=field_options(alias="Containers"))
    containers_running: int | None = field(default=None, metadata=field_options(alias="ContainersRunning"))
    containers_paused: int | None = field(default=None, metadata=field_options(alias="ContainersPaused"))
    containers_stopped: int | None = field(default=None, metadata=field_options(alias="ContainersStopped"))
    images: int | None = field(default=None, metadata=field_options(alias="Images"))
    driver: str | None = field(default=None, metadata=field_options(alias="Driver"))
    driver_status: list[list[str]] | None = field(default=None, metadata=field_options(alias="DriverStatus"))
    docker_root_dir: str | None = field(default=None, metadata=field_options(alias="DockerRootDir"))
    plugins: PluginsInfo | None = field(default=None, metadata=field_options(alias="Plugins"))
    memory_limit: bool | None = field(default=None, metadata=field_options(alias="MemoryLimit"))
    swap_limit: bool | None = field(default=None, metadata=field_options(alias="SwapLimit"))
    kernel_memory_tcp: bool | None = field(default=None, metadata=field_options(alias="KernelMemoryTCP"))
    cpu_cfs_period: bool | None = field(default=None, metadata=field_options(alias="CpuCfsPeriod"))
    cpu_cfs_quota: bool | None = field(default=None, metadata=field_options(alias="CpuCfsQuota"))
    cpu_shares: bool | None = field(default=None, metadata=field_options(alias="CPUShares"))
    cpu_set: bool | None = field(default=None, metadata=field_options(alias="CPUSet"))
    pids_limit: bool | None = field(default=None, metadata=field_options(alias="PidsLimit"))
    oom_kill_disable: bool | None = field(default=None, metadata=field_options(alias="OomKillDisable"))
    ipv4_forwarding: bool | None = field(default=None, metadata=field_options(alias="IPv4Forwarding"))
    bridge_nf_iptables: bool | None = field(default=None, metadata=field_options(alias="BridgeNfIptables"))
    bridge_nf_ip6tables: bool | None = field(default=None, metadata=field_options(alias="BridgeNfIp6tables"))
    debug: bool | None = field(default=None, metadata=field_options(alias="Debug"))
    nfd: int | None = field(default=None, metadata=field_options(alias="NFd"))
    ngoroutines: int | None = field(default=None, metadata=field_options(alias="NGoroutines"))
    system_time: str | None = field(default=None, metadata=field_options(alias="SystemTime"))
    logging_driver: str | None = field(default=None, metadata=field_options(alias="LoggingDriver"))
    cgroup_driver: str | None = field(default=None, metadata=field_options(alias="CgroupDriver"))
    cgroup_version: str | None = field(default=None, metadata=field_options(alias="CgroupVersion"))
    nevents_listener: int | None = field(default=None, metadata=field_options(alias="NEventsListener"))
    kernel_version: str | None = field(default=None, metadata=field_options(alias="KernelVersion"))
    operating_system: str | None = field(default=None, metadata=field_options(alias="OperatingSystem"))
    os_version: str | None = field(default=None, metadata=field_options(alias="OSVersion"))
    os_type: str | None = field(default=None, metadata=field_options(alias="OSType"))
    architecture: str | None = field(default=None, metadata=field_options(alias="Architecture"))
    ncpu: int | None = field(default=None, metadata=field_options(alias="NCPU"))
    mem_total: int | None = field(default=None, metadata=field_options(alias="MemTotal"))
    index_server_address: str | None = field(default=None, metadata=field_options(alias="IndexServerAddress"))
    registry_config: RegistryConfig | None = field(default=None, metadata=field_options(alias="RegistryConfig"))
    generic_resources: list[dict[str, Any]] | None = field(default=None, metadata=field_options(alias="GenericResources"))
    http_proxy: str | None = field(default=None, metadata=field_options(alias="HttpProxy"))
    https_proxy: str | None = field(default=None, metadata=field_options(alias="HttpsProxy"))
    no_proxy: str | None = field(default=None, metadata=field_options(alias="NoProxy"))
    name: str | None = field(default=None, metadata=field_options(alias="Name"))
    labels: list[str] | None = field(default=None, metadata=field_options(alias="Labels"))
    experimental_build: bool | None = field(default=None, metadata=field_options(alias="ExperimentalBuild"))
    server_version: str | None = field(default=None, metadata=field_options(alias="ServerVersion"))
    runtimes: dict[str, Any] | None = field(default=None, metadata=field_options(alias="Runtimes"))
    default_runtime: str | None = field(default=None, metadata=field_options(alias="DefaultRuntime"))
    swarm: SwarmInfo | None = field(default=None, metadata=field_options(alias="Swarm"))
    live_restore_enabled: bool | None = field(default=None, metadata=field_options(alias="LiveRestoreEnabled"))
    isolation: str | None = field(default=None, metadata=field_options(alias="Isolation"))
    init_binary: str | None = field(default=None, metadata=field_options(alias="InitBinary"))
    containerd_commit: CommitInfo | None = field(default=None, metadata=field_options(alias="ContainerdCommit"))
    runc_commit: CommitInfo | None = field(default=None, metadata=field_options(alias="RuncCommit"))
    init_commit: CommitInfo | None = field(default=None, metadata=field_options(alias="InitCommit"))
    security_options: list[str] | None = field(default=None, metadata=field_options(alias="SecurityOptions"))
    product_license: str | None = field(default=None, metadata=field_options(alias="ProductLicense"))
    default_address_pools: list[dict[str, Any]] | None = field(default=None, metadata=field_options(alias="DefaultAddressPools"))
    warnings: list[str] | None = field(default=None, metadata=field_options(alias="Warnings"))



"""Models for Portainer API."""

from __future__ import annotations

from dataclasses import dataclass, field
from typing import Any

from mashumaro import field_options
from mashumaro.mixins.orjson import DataClassORJSONMixin


@dataclass
class KubernetesSnapshot(DataClassORJSONMixin):
    """Represents a Kubernetes snapshot, including diagnostics data, version, node count, and resource usage."""

    diagnostics_data: dict[str, str] | None = field(default=None, metadata=field_options(alias="DiagnosticsData"))
    kubernetes_version: str | None = field(default=None, metadata=field_options(alias="KubernetesVersion"))
    node_count: int | None = field(default=None, metadata=field_options(alias="NodeCount"))
    time: int | None = field(default=None, metadata=field_options(alias="Time"))
    total_cpu: int | None = field(default=None, metadata=field_options(alias="TotalCPU"))
    total_memory: int | None = field(default=None, metadata=field_options(alias="TotalMemory"))


@dataclass
class TLSConfig(DataClassORJSONMixin):
    """Represents TLS configuration."""

    tls: bool | None = field(default=None, metadata=field_options(alias="TLS"))
    tls_ca_cert: str | None = field(default=None, metadata=field_options(alias="TLSCACert"))
    tls_cert: str | None = field(default=None, metadata=field_options(alias="TLSCert"))
    tls_key: str | None = field(default=None, metadata=field_options(alias="TLSKey"))
    tls_skip_verify: bool | None = field(default=None, metadata=field_options(alias="TLSSkipVerify"))


@dataclass
class SecuritySettings(DataClassORJSONMixin):
    """Represents security settings for an endpoint."""

    allow_bind_mounts_for_regular_users: bool | None = field(default=None, metadata=field_options(alias="allowBindMountsForRegularUsers"))
    allow_container_capabilities_for_regular_users: bool | None = field(
        default=None, metadata=field_options(alias="allowContainerCapabilitiesForRegularUsers")
    )
    allow_device_mapping_for_regular_users: bool | None = field(default=None, metadata=field_options(alias="allowDeviceMappingForRegularUsers"))
    allow_host_namespace_for_regular_users: bool | None = field(default=None, metadata=field_options(alias="allowHostNamespaceForRegularUsers"))
    allow_privileged_mode_for_regular_users: bool | None = field(default=None, metadata=field_options(alias="allowPrivilegedModeForRegularUsers"))
    allow_stack_management_for_regular_users: bool | None = field(default=None, metadata=field_options(alias="allowStackManagementForRegularUsers"))
    allow_sysctl_setting_for_regular_users: bool | None = field(default=None, metadata=field_options(alias="allowSysctlSettingForRegularUsers"))
    allow_volume_browser_for_regular_users: bool | None = field(default=None, metadata=field_options(alias="allowVolumeBrowserForRegularUsers"))
    enable_host_management_features: bool | None = field(default=None, metadata=field_options(alias="enableHostManagementFeatures"))


@dataclass
class Agent(DataClassORJSONMixin):
    """Represents agent information."""

    version: str | None = field(default=None, metadata=field_options(alias="version"))


@dataclass
class Edge(DataClassORJSONMixin):
    """Represents edge configuration."""

    command_interval: int | None = field(default=None, metadata=field_options(alias="CommandInterval"))
    ping_interval: int | None = field(default=None, metadata=field_options(alias="PingInterval"))
    snapshot_interval: int | None = field(default=None, metadata=field_options(alias="SnapshotInterval"))
    async_mode: bool | None = field(default=None, metadata=field_options(alias="asyncMode"))


@dataclass
class Endpoint(DataClassORJSONMixin):  # pylint: disable=too-many-instance-attributes
    """Represents a Portainer endpoint."""

    id: int = field(metadata=field_options(alias="Id"))

    amt_device_guid: str | None = field(default=None, metadata=field_options(alias="AMTDeviceGUID"))
    authorized_teams: list[int] | None = field(default=None, metadata=field_options(alias="AuthorizedTeams"))
    authorized_users: list[int] | None = field(default=None, metadata=field_options(alias="AuthorizedUsers"))
    azure_credentials: dict[str, str] | None = field(default=None, metadata=field_options(alias="AzureCredentials"))
    compose_syntax_max_version: str | None = field(default=None, metadata=field_options(alias="ComposeSyntaxMaxVersion"))
    container_engine: str | None = field(default=None, metadata=field_options(alias="ContainerEngine"))
    edge_checkin_interval: int | None = field(default=None, metadata=field_options(alias="EdgeCheckinInterval"))
    edge_id: str | None = field(default=None, metadata=field_options(alias="EdgeID"))
    edge_key: str | None = field(default=None, metadata=field_options(alias="EdgeKey"))
    enable_gpu_management: bool | None = field(default=None, metadata=field_options(alias="EnableGPUManagement"))
    gpus: list[dict[str, str]] | None = field(default=None, metadata=field_options(alias="Gpus"))
    group_id: int | None = field(default=None, metadata=field_options(alias="GroupId"))
    heartbeat: bool | None = field(default=None, metadata=field_options(alias="Heartbeat"))
    is_edge_device: bool | None = field(default=None, metadata=field_options(alias="IsEdgeDevice"))
    kubernetes: dict[str, Any] | None = field(default=None, metadata=field_options(alias="Kubernetes"))
    name: str | None = field(default=None, metadata=field_options(alias="Name"))
    public_url: str | None = field(default=None, metadata=field_options(alias="PublicURL"))
    snapshots: list[dict[str, Any]] | None = field(default=None, metadata=field_options(alias="Snapshots"))
    status: int | None = field(default=None, metadata=field_options(alias="Status"))
    tls: bool | None = field(default=None, metadata=field_options(alias="TLS"))
    tls_ca_cert: str | None = field(default=None, metadata=field_options(alias="TLSCACert"))
    tls_cert: str | None = field(default=None, metadata=field_options(alias="TLSCert"))
    tls_key: str | None = field(default=None, metadata=field_options(alias="TLSKey"))
    tag_ids: list[int] | None = field(default=None, metadata=field_options(alias="TagIds"))
    tags: list[str] | None = field(default=None, metadata=field_options(alias="Tags"))
    team_access_policies: dict[str, dict[str, int]] | None = field(default=None, metadata=field_options(alias="TeamAccessPolicies"))
    type: int | None = field(default=None, metadata=field_options(alias="Type"))
    url: str | None = field(default=None, metadata=field_options(alias="URL"))
    user_access_policies: dict[str, dict[str, int]] | None = field(default=None, metadata=field_options(alias="UserAccessPolicies"))
    user_trusted: bool | None = field(default=None, metadata=field_options(alias="UserTrusted"))
    tls_config: TLSConfig | None = field(default=None, metadata=field_options(alias="TLSConfig"))
    agent: Agent | None = field(default=None, metadata=field_options(alias="agent"))
    edge: Edge | None = field(default=None, metadata=field_options(alias="edge"))
    last_check_in_date: int | None = field(default=None, metadata=field_options(alias="lastCheckInDate"))
    query_date: int | None = field(default=None, metadata=field_options(alias="queryDate"))
    security_settings: SecuritySettings | None = field(default=None, metadata=field_options(alias="securitySettings"))



"""Asynchronous Python client for Python Portainer."""

from .exceptions import (
    PortainerAuthenticationError,
    PortainerConnectionError,
    PortainerError,
    PortainerTimeoutError,
)
from .pyportainer import Portainer

__all__ = [
    "Portainer",
    "PortainerAuthenticationError",
    "PortainerConnectionError",
    "PortainerError",
    "PortainerTimeoutError",
]



"""Asynchronous Python client for Python Portainer."""


class PortainerError(Exception):
    """Generic exception for Portainer errors."""


class PortainerConnectionError(PortainerError):
    """Exception raised for connection errors."""


class PortainerTimeoutError(PortainerError):
    """Exception raised for timeout errors."""


class PortainerAuthenticationError(PortainerError):
    """Exception raised for authentication errors."""


class PortainerNotFoundError(PortainerError):
    """Exception raised when a resource is not found."""
